package ca.recrown.islandsurvivalcraft.world.shaders;

import java.util.Random;

import org.bukkit.block.Biome;
import org.bukkit.util.noise.SimplexOctaveGenerator;

import ca.recrown.islandsurvivalcraft.world.IslandWorldMapper;

public class WorldHeightShader {
    private final IslandWorldMapper mapper;
    private final SimplexOctaveGenerator shader;
    private final int seaLevel;
    private final int worldHeight;
    private final int minimumHeight;

    public WorldHeightShader(long seed, IslandWorldMapper islandLocator, int seaLevel, int worldHeight, int minimumHeight) {
        this.mapper = islandLocator;
        this.seaLevel = seaLevel;
        this.worldHeight = worldHeight;
        this.minimumHeight = minimumHeight;
        this.shader = new SimplexOctaveGenerator(new Random(seed - 1), 8);
        this.shader.setScale(0.05d);
    }

    public int getTerrainHeight(int worldX, int worldZ, Biome biome) {
        int height = 0;
        String biomeName = biome.name().toLowerCase();
        if (biomeName.contains("hills")) {
            height = calculateTerrainHeight(worldX, worldZ, 30d);
        } else if (biomeName.contains("mountains")) {
            height = calculateTerrainHeight(worldX, worldZ, 70d);
        } else if (biomeName.contains("plateau")) {
            height = (int) Math.min(calculateTerrainHeight(worldX, worldZ, 50d), seaLevel + 30d);
        } else if (biomeName.contains("modified")) {
            height = calculateTerrainHeight(worldX, worldZ, 40d);
        } else if (biomeName.contains("shattered")) {
            height = calculateTerrainHeight(worldX, worldZ, 40d);
        } else if (biomeName.contains("tall")) {
            height = calculateTerrainHeight(worldX, worldZ, 30d);
        } else if (biomeName.contains("stone")) {
            height = calculateTerrainHeight(worldX, worldZ, 70d);
        } else if (!biomeName.contains("ocean")) {
            height = calculateTerrainHeight(worldX, worldZ, 10d);
        } else {
            height = calculateTerrainHeight(worldX, worldZ, seaLevel, 1d, 0.25d);
        }

        if (height > worldHeight) throw new IllegalStateException("Resulting height is greater than world height! Biome this occurred on: " + biomeName);
        height = Math.max(minimumHeight, height);
        return height;
    }

    private int calculateTerrainHeight(int worldX, int worldZ, double heightFactor, double shift, double exaggerationFactor) {
        int res = seaLevel;
        double shapeValue = (shader.noise(worldX, worldZ, 1.5d, 0.05d, true) + shift) / (shift + 1d);
        if (shift == 1d && exaggerationFactor != 1d) shapeValue = Math.pow(shapeValue, exaggerationFactor);
        res += mapper.getWorldValue(worldX, worldZ) * (shapeValue * heightFactor);
        return res;
    }

    private int calculateTerrainHeight(int worldX, int worldZ, double heightFactor) {
        return calculateTerrainHeight(worldX, worldZ, heightFactor, 0.75d, 1d);
    }
 }