package ca.recrown.islandsurvivalcraft.world.shaders;

import java.util.Random;

import org.bukkit.Material;
import org.bukkit.block.Biome;

import ca.recrown.islandsurvivalcraft.world.IslandWorldMapper;

public class WorldLayerShader {
    private final IslandWorldMapper mapper;
    private final long seed;
    private final int seaLevel, maxHeight;
    private final Random random = new Random();


    private final Material[] badlandTerrocota = new Material[] {
        Material.BLACK_TERRACOTTA, 
        Material.BROWN_TERRACOTTA, 
        Material.GRAY_TERRACOTTA, 
        Material.LIGHT_GRAY_TERRACOTTA, 
        Material.YELLOW_TERRACOTTA, 
        Material.WHITE_TERRACOTTA
    };

    public WorldLayerShader(long seed, int seaLevel, int maxHeight, IslandWorldMapper mapper) {
        this.mapper = mapper;
        this.seed = seed;
        this.seaLevel = seaLevel;
        this.maxHeight = maxHeight;
    }

    public boolean hasSpecialLayers(Biome biome) {
        String biomeName = biome.toString().toLowerCase();
        return 
        biomeName.contains("badlands");
    }

    /**
     * Figures out the type of material to be placed at a given world coordinate set.
     * Returns null when special layering is done and should return to normal generation.
     * @param worldX The world x coordinate.
     * @param worldZ The world z coordinate.
     * @param y The elevation.
     * @param biome the associated biome.
     * @return the material for the layer. Returning null means no more special layers.
     */
    public Material getMaterialForHeight(int worldX, int worldZ, int y, Biome biome) {
        String biomeName = biome.toString().toLowerCase();
        if (biomeName.contains("badlands")) {
            int seedOffset = (worldX + worldZ) / 1000;
            random.setSeed(seed + seedOffset);
            int yInterval = (int) (random.nextFloat() * 9) + 1;
            int yBandInitial = (int) (seaLevel + random.nextDouble() * 50);
            
            if (y > yBandInitial) {
                int amountOfLayers = Math.min(Math.max(1, random.nextInt(badlandTerrocota.length)), 4);
                int subLayer = y % yInterval;
                if (subLayer < amountOfLayers) {
                    Material[] selected = new Material[amountOfLayers];
                    for (int i = 0; i < selected.length; i++) {
                        selected[i] = badlandTerrocota[random.nextInt(amountOfLayers)];
                    }
                    return selected[subLayer];
                } else {
                    return Material.TERRACOTTA;
                }
            }
        }
        return null;
    }

    public Material getSurfaceMaterial(Biome biome) {
        String biomeName = biome.toString().toLowerCase();
        if (biomeName.contains("beach") || biomeName.contains("desert") || biomeName.contains("warm_ocean")) {
            return Material.SAND;
        } else if (biomeName.contains("ocean")) {
            return Material.GRAVEL;
        } else if (biomeName.contains("stone")) {
            return Material.STONE;
        }

        return Material.GRASS_BLOCK;
    }

    public int getSurfaceThickness(int worldX, int worldZ, Biome biome) {
        String biomeName = biome.toString().toLowerCase();

        if (biomeName.contains("beach")) {
            return (int) (Math.abs(mapper.getWorldValue(worldX, worldZ)) * 5) + 4;
        }

        return 1;
    }
    
    public Material getTransitionMaterial(Biome biome) {
        String biomeName = biome.toString().toLowerCase();
        if (biomeName.contains("beach") || biomeName.contains("desert")) {
            return Material.SANDSTONE;
        } else if (biomeName.contains("stone")) {
            return Material.STONE;
        } else if (biomeName.contains("ocean")) {
            return Material.GRAVEL;
        }
        return Material.DIRT;
    }
    
    public int getTransitionMaterialThickness(int worldX, int worldZ, Biome biome) {
        return (int) (Math.abs(mapper.getWorldValue(worldX, worldZ)) * 10) + 6;
    }
}